@@ -8,7 +8,7 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -32,12 +32,18 @@
 
 #include <stdarg.h>
 #include <glib.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
+/* Offsets for seeking forward, and rewinding in seconds */
+#define FFWD_OFFSET 20
+#define RWD_OFFSET 10
+
+#define RB_CHAIN_GOBJECT_METHOD(klass, method, instance) \
+	if (G_OBJECT_CLASS (klass)->method != NULL) \
+		G_OBJECT_CLASS (klass)->method (instance);
+
 gboolean rb_true_function (gpointer dummy);
 gboolean rb_false_function (gpointer dummy);
 gpointer rb_null_function (gpointer dummy);
@@ -54,12 +60,6 @@ void rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path);
 
 GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size);
 
-gchar *rb_uri_get_mount_point (const char *uri);
-gchar *rb_uri_get_filesystem_type(const char *uri);
-
-gboolean rb_uri_is_mounted (const char *uri);
-
-
 void rb_threads_init (void);
 gboolean rb_is_main_thread (void);
 
@@ -80,12 +80,16 @@ GList* rb_collate_hash_table_keys (GHashTable *table);
 GList* rb_collate_hash_table_values (GHashTable *table);
 
 GList* rb_uri_list_parse (const char *uri_list);
-const gchar* rb_mime_get_friendly_name (const gchar *mime_type);
+char* rb_mime_get_friendly_name (const char *mime_type);
 
 gboolean rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
 					       GValue *return_accu,
 					       const GValue *handler_return,
 					       gpointer dummy);
+gboolean rb_signal_accumulator_value_array (GSignalInvocationHint *hint,
+					    GValue *return_accu,
+					    const GValue *handler_return,
+					    gpointer dummy);
 void rb_value_array_append_data (GValueArray *array, GType type, ...);
 void rb_value_free (GValue *val); /* g_value_unset, g_slice_free */
 
