@@ -67,6 +67,7 @@ typedef enum
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID,     /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID,   /* string */
 	RB_METADATA_FIELD_ARTIST_SORTNAME,         /* string */
+	RB_METADATA_FIELD_ALBUM_SORTNAME,         /* string */
 
 	RB_METADATA_FIELD_LAST			   /* nothing */
 } RBMetaDataField;
@@ -79,8 +80,6 @@ typedef enum
 	RB_METADATA_ERROR_UNSUPPORTED,
 	RB_METADATA_ERROR_GENERAL,
 	RB_METADATA_ERROR_INTERNAL,
-	RB_METADATA_ERROR_NOT_AUDIO,
-	RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
 	RB_METADATA_ERROR_EMPTY_FILE
 } RBMetaDataError;
 
@@ -121,6 +120,7 @@ const char *	rb_metadata_get_field_name (RBMetaDataField field);
 RBMetaData *	rb_metadata_new		(void);
 
 gboolean	rb_metadata_can_save	(RBMetaData *md, const char *mimetype);
+char **		rb_metadata_get_saveable_types (RBMetaData *md);
 
 void		rb_metadata_load	(RBMetaData *md,
 					 const char *uri,
@@ -136,6 +136,9 @@ gboolean	rb_metadata_has_missing_plugins (RBMetaData *md);
 gboolean	rb_metadata_get_missing_plugins (RBMetaData *md,
 					 char ***missing_plugins,
 					 char ***plugin_descriptions);
+gboolean	rb_metadata_has_audio	(RBMetaData *md);
+gboolean	rb_metadata_has_video	(RBMetaData *md);
+gboolean	rb_metadata_has_other_data (RBMetaData *md);
 
 gboolean	rb_metadata_get		(RBMetaData *md, RBMetaDataField field,
 					 GValue *val);
