@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -222,6 +230,44 @@ rhythmdb_query_append (RhythmDB *db, GPtrArray *query, ...)
 	va_end (args);
 }
 
+void
+rhythmdb_query_append_params (RhythmDB *db, GPtrArray *query,
+			      RhythmDBQueryType type, RhythmDBPropType prop, const GValue *value)
+{
+	RhythmDBQueryData *data = g_new0 (RhythmDBQueryData, 1);
+
+	data->type = type;
+	switch (type) {
+	case RHYTHMDB_QUERY_END:
+		g_assert_not_reached ();
+		break;
+	case RHYTHMDB_QUERY_DISJUNCTION:
+		break;
+	case RHYTHMDB_QUERY_SUBQUERY:
+		data->subquery = rhythmdb_query_copy (g_value_get_pointer (value));
+		break;
+	case RHYTHMDB_QUERY_PROP_EQUALS:
+	case RHYTHMDB_QUERY_PROP_LIKE:
+	case RHYTHMDB_QUERY_PROP_NOT_LIKE:
+	case RHYTHMDB_QUERY_PROP_PREFIX:
+	case RHYTHMDB_QUERY_PROP_SUFFIX:
+	case RHYTHMDB_QUERY_PROP_GREATER:
+	case RHYTHMDB_QUERY_PROP_LESS:
+	case RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN:
+	case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
+	case RHYTHMDB_QUERY_PROP_YEAR_EQUALS:
+	case RHYTHMDB_QUERY_PROP_YEAR_GREATER:
+	case RHYTHMDB_QUERY_PROP_YEAR_LESS:
+		data->propid = prop;
+		data->val = g_new0 (GValue, 1);
+		g_value_init (data->val, rhythmdb_get_property_type (db, data->propid));
+		g_value_copy (value, data->val);
+		break;
+	}
+
+	g_ptr_array_add (query, data);
+}
+
 /**
  * rhythmdb_query_free:
  * @query: a query.
@@ -349,7 +395,19 @@ rhythmdb_read_encoded_property (RhythmDB *db,
 		g_value_set_uint64 (val, g_ascii_strtoull (content, NULL, 10));
 		break;
 	case G_TYPE_DOUBLE:
-		g_value_set_double (val, g_ascii_strtod (content, NULL));
+		{
+			gdouble d;
+			char *end;
+
+			d = g_ascii_strtod (content, &end);
+			if (*end != '\0') {
+				/* conversion wasn't entirely successful.
+				 * try locale-aware strtod().
+				 */
+				d = strtod (content, NULL);
+			}
+			g_value_set_double (val, d);
+		}
 		break;
 	case G_TYPE_POINTER:
 		if (propid == RHYTHMDB_PROP_TYPE) {
@@ -622,18 +680,25 @@ rhythmdb_query_preprocess (RhythmDB *db, GPtrArray *query)
 				gulong year;
 
 				search_date = g_value_get_ulong (data->val);
-				g_date_set_julian (&date, search_date);
-				year = g_date_get_year (&date);
-				g_date_clear (&date, 1);
+				
+				/* GDate functions don't handle Year="0", so we need to special case this */
+				if (search_date != 0) {
+					g_date_set_julian (&date, search_date);
+					year = g_date_get_year (&date);
+					g_date_clear (&date, 1);
 
-				/* get Julian dates for beginning and end of year */
-				g_date_set_dmy (&date, 1, G_DATE_JANUARY, year);
-				begin = g_date_get_julian (&date);
-				g_date_clear (&date, 1);
+					/* get Julian dates for beginning and end of year */
+					g_date_set_dmy (&date, 1, G_DATE_JANUARY, year);
+					begin = g_date_get_julian (&date);
+					g_date_clear (&date, 1);
 
-				/* and the day before the beginning of the next year */
-				g_date_set_dmy (&date, 1, G_DATE_JANUARY, year + 1);
-				end =  g_date_get_julian (&date) - 1;
+					/* and the day before the beginning of the next year */
+					g_date_set_dmy (&date, 1, G_DATE_JANUARY, year + 1);
+					end =  g_date_get_julian (&date) - 1;
+				} else {
+					begin = 0;
+					end = 0;
+				}
 
 				switch (data->type)
 				{
