@@ -70,9 +70,7 @@
 
 static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
-static GObject *rb_library_source_constructor (GType type,
-					       guint n_construct_properties,
-					       GObjectConstructParam *construct_properties);
+static void rb_library_source_constructed (GObject *object);
 static void rb_library_source_dispose (GObject *object);
 static void rb_library_source_finalize (GObject *object);
 
@@ -188,7 +186,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	object_class->dispose = rb_library_source_dispose;
 	object_class->finalize = rb_library_source_finalize;
-	object_class->constructor = rb_library_source_constructor;
+	object_class->constructed = rb_library_source_constructed;
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_config_widget = impl_get_config_widget;
@@ -287,18 +285,16 @@ db_load_complete_cb (RhythmDB *db, RBLibrarySource *source)
 	g_object_set (source, "populate", TRUE, NULL);
 }
 
-static GObject *
-rb_library_source_constructor (GType type,
-			       guint n_construct_properties,
-			       GObjectConstructParam *construct_properties)
+static void
+rb_library_source_constructed (GObject *object)
 {
 	RBLibrarySource *source;
 	RBShell *shell;
 	RBEntryView *songs;
 	GSList *list;
 
-	source = RB_LIBRARY_SOURCE (G_OBJECT_CLASS (rb_library_source_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_library_source_parent_class, constructed, object);
+	source = RB_LIBRARY_SOURCE (object);
 
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &source->priv->db, NULL);
@@ -359,8 +355,6 @@ rb_library_source_constructor (GType type,
 	g_idle_add ((GSourceFunc)add_child_sources_idle, source);
 
 	g_object_unref (shell);
-
-	return G_OBJECT (source);
 }
 
 /**
@@ -1026,6 +1020,7 @@ rb_library_source_layout_path_changed (GConfClient *client,
 	char *value;
 	int i = 0;
 
+	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_PATH) == 0);
 
 	rb_debug ("layout path changed");
@@ -1057,6 +1052,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 	char *value;
 	int i = 0;
 
+	g_return_if_fail (entry != NULL);
 	g_return_if_fail (strcmp (entry->key, CONF_LIBRARY_LAYOUT_FILENAME) == 0);
 
 	rb_debug ("layout filename changed");
@@ -1204,9 +1200,17 @@ impl_can_paste (RBSource *asource)
 }
 
 static void
-completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, RBLibrarySource *source)
+completed_cb (RhythmDBEntry *entry, const char *dest, GError *error, RBLibrarySource *source)
 {
-	rhythmdb_add_uri (source->priv->db, dest);
+	if (error == NULL) {
+		rhythmdb_add_uri (source->priv->db, dest);
+	} else {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+			rb_debug ("not displaying 'file exists' error for %s", dest);
+		} else {
+			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+		}
+	}
 }
 
 static void
